@@ -1690,6 +1690,7 @@ decompileSETMEMBER(int n, SWF_ACTION *actions, int maxn)
 	val = pop();
 	var = pop();
 	obj = pop();
+
 #ifdef DEBUG
 	printf("*SETMember* varName %s (type=%d)  objName=%s (type=%d)\n",getName(var),var->Type, getName(obj),obj->Type);
 #endif
@@ -1729,10 +1730,20 @@ decompileSETMEMBER(int n, SWF_ACTION *actions, int maxn)
 		puts("]");
 	}
 	printf(" = " );
-	if (val->Type != PUSH_VARIABLE)			// later it will be a switch{}
+
+
+	if ( OpCode(actions, n-1, maxn) == SWFACTION_STOREREGISTER ) {
+		struct SWF_ACTIONSTOREREGISTER *sr =
+			(struct SWF_ACTIONSTOREREGISTER*)&actions[n-1];
+		printf("R%d", sr->Register);
+	}
+	else if (val->Type != PUSH_VARIABLE) {
+		/* later it will be a switch{} */
 		decompilePUSHPARAM(val,1);
-	else
+	}
+	else {
 		decompilePUSHPARAM(val,0);
+	}
 	println(";");
 	return 0;
 }
@@ -3033,11 +3044,24 @@ decompileEXTENDS(int n, SWF_ACTION *actions, int maxn)
 	struct SWF_ACTIONPUSHPARAM *baseclass;
 
 	baseclass=pop();
+#if 0
+	/* It's useless to open a class body when there's no
+	 * other code supporting it. */
 	printf("class ");
 	puts(getName(pop()));
 	printf(" extends ");
 	puts(getName(baseclass));
 	println(" {" );
+#else
+	/* We'll do it with asm{} */
+	println("asm {");
+	println(" push '%s'", getName(pop()));
+	println(" getvariable");
+	println(" push '%s'", getName(baseclass));
+	println(" getvariable");
+	println(" extends");
+	println("};");
+#endif
 
 	return 0;
 }
