@@ -135,7 +135,7 @@ static char* base_name(char* filename);
 static char **import_specs;
 static int numimport_specs = 0;
 static int swfversion = DEFSWFVERSION;
-static const char *RCSID = "$Id: makeswf.c,v 1.50 2009/09/08 22:26:16 strk Exp $";
+static const char *RCSID = "$Id$";
 static SWFMovie mo;
 static int useBgColor=0;
 static long int bgcolor=0;
@@ -715,9 +715,26 @@ embed_swf(SWFMovie movie, char* filename)
 	SWFPrebuiltClip builtclip;
 	SWFDisplayItem it;
 	char *name;
+	SWFInput in;
+	FILE *swf;
 
-	builtclip = newSWFPrebuiltClip_fromFile(filename);
-	if ( ! builtclip ) exit (1);
+        if (!(swf = fopen (filename, "rb")))
+        {
+                fprintf (stdout, "%s: %s\n", filename, strerror (errno));
+		exit(1);
+        }
+
+        if (!(in = newSWFInput_file(swf)))
+        {
+                fprintf (stdout, "Can't create SWFInput from file\n");
+		exit(1);
+        }
+
+	builtclip = newSWFPrebuiltClip_fromInput(in);
+	if ( ! builtclip ) {
+                fprintf (stdout, "Error creating prebuilt clip\n");
+		exit(1);
+	}
 
 	it = SWFMovie_add(mo, (SWFBlock)builtclip);
 
@@ -728,7 +745,7 @@ embed_swf(SWFMovie movie, char* filename)
 
 /**************************************************************
  *
- * $Log: makeswf.c,v $
+ * $Log$
  * Revision 1.50  2009/09/08 22:26:16  strk
  * Update copyright notice (and FSF address)
  *
