@@ -126,7 +126,7 @@ struct FONTINFO {		/* a linked list for all our font code info: */
  struct FONTINFO *next;
 };
 static struct FONTINFO *fip; 	/* start point of list */
-
+static struct FONTINFO *fip_current; 
 #define OUT_BEGIN(block) \
 	struct block *sblock = (struct block *)pblock; \
 	printf( "\n" COMMSTART " " #block " " COMMEND "\n");
@@ -148,6 +148,7 @@ static struct SWFBlockOutput outputs[] = {
   {SWF_DEFINEEDITTEXT, outputSWF_DEFINEEDITTEXT},
   {SWF_DEFINEFONT, outputSWF_DEFINEFONT},
   {SWF_DEFINEFONT2, outputSWF_DEFINEFONT2},
+  {SWF_DEFINEFONT3, outputSWF_DEFINEFONT3},
   {SWF_DEFINEFONTINFO, outputSWF_DEFINEFONTINFO},
   {SWF_DEFINEFONTINFO2, outputSWF_DEFINEFONTINFO2},
   {SWF_DEFINELOSSLESS, outputSWF_DEFINELOSSLESS},
@@ -390,13 +391,13 @@ outputSWF_CXFORMWITHALPHA(SWF_CXFORMWITHALPHA * cxform, char *name)
  if (cxform->HasMultTerms)
  {
   printf("%s" ARGSTART "%0.2f" ARGSEP "%0.2f" ARGSEP "%0.2f" ARGSEP "%0.2f" ARGEND STMNTEND "\n",
-    methodcall (name, "setColorMult"), cxform->RedMultTerm/256.0,
+    methodcall (name, "multColor"), cxform->RedMultTerm/256.0,
     cxform->GreenMultTerm/256.0, cxform->BlueMultTerm/256.0, cxform->AlphaMultTerm/256.0);
  }
  if (cxform->HasAddTerms)
  {
   printf("%s" ARGSTART "%ld" ARGSEP "%ld" ARGSEP "%ld" ARGSEP "%ld" ARGEND STMNTEND "\n",
-    methodcall (name, "setColorAdd"),  cxform->RedAddTerm,
+    methodcall (name, "addColor"),  cxform->RedAddTerm,
     cxform->GreenAddTerm, cxform->BlueAddTerm, cxform->AlphaAddTerm);
  }	
 }
@@ -576,10 +577,13 @@ outputSWF_LINESTYLEARRAY (SWF_LINESTYLEARRAY * linestylearray,
 
   count = linestylearray->LineStyleCount;
 
+  SWF_LINESTYLE* styles = linestylearray->LineStyles ?
+  	linestylearray->LineStyles : linestylearray->LineStyles2;
+
   printf ("" COMMSTART "%d linestyles(s)" COMMEND "\n", count);
   for (i = 0; i < count; i++)
     {
-      outputSWF_LINESTYLE (&(linestylearray->LineStyles[i]), parentname, i);
+      outputSWF_LINESTYLE (&(styles[i]), parentname, i);
     }
 }
 
@@ -724,9 +728,9 @@ outputSWF_DEFINEBITS (SWF_Parserstruct * pblock)
   OUT_BEGIN (SWF_DEFINEBITS);
 
   sprintf (name, "character%d", sblock->CharacterID);
-  printf ("\n" COMMSTART " Bitmap %d (bits). To extract:" COMMEND "\n",
+  printf ("\n" COMMSTART " Bitmap %d (bits). To extract: " COMMEND "\n",
     sblock->CharacterID);
-  printf ("\n" COMMSTART " swfextract -j %d -o %s.jpg $swf" COMMEND "\n",
+  printf (COMMSTART " swfextract -j %d -o %s.jpg $swf " COMMEND "\n",
     sblock->CharacterID, name);
   printf ("%s('%s.jpg');\n", newobj (name, "Bitmap"), name);
 
@@ -742,9 +746,9 @@ outputSWF_DEFINEBITSJPEG2 (SWF_Parserstruct * pblock)
   OUT_BEGIN (SWF_DEFINEBITSJPEG2);
 
   sprintf (name, "character%d", sblock->CharacterID);
-  printf ("\n" COMMSTART " Bitmap %d (jpeg2). To extract:" COMMEND "\n",
+  printf ("\n" COMMSTART " Bitmap %d (jpeg2). To extract: " COMMEND "\n",
     sblock->CharacterID);
-  printf ("\n" COMMSTART " swfextract -j %d -o %s.jpg $swf" COMMEND "\n",
+  printf (COMMSTART " swfextract -j %d -o %s.jpg $swf " COMMEND "\n",
     sblock->CharacterID, name);
   printf ("%s('%s.jpg');\n", newobj (name, "Bitmap"), name);
 
@@ -760,9 +764,9 @@ outputSWF_DEFINEBITSJPEG3 (SWF_Parserstruct * pblock)
   OUT_BEGIN (SWF_DEFINEBITSJPEG3);
 
   sprintf (name, "character%d", sblock->CharacterID);
-  printf ("\n" COMMSTART " Bitmap %d (jpeg3). To extract:" COMMEND "\n",
+  printf ("\n" COMMSTART " Bitmap %d (jpeg3). To extract: " COMMEND "\n",
     sblock->CharacterID);
-  printf ("\n" COMMSTART " swfextract -j %d -o %s.jpg $swf" COMMEND "\n",
+  printf (COMMSTART " swfextract -j %d -o %s.jpg $swf " COMMEND "\n",
     sblock->CharacterID, name);
   printf ("%s('%s.jpg');\n", newobj (name, "Bitmap"), name);
 }
@@ -909,8 +913,9 @@ outputSWF_DEFINEEDITTEXT (SWF_Parserstruct * pblock)
   OUT_BEGIN (SWF_DEFINEEDITTEXT);
 
   sprintf (tname, "character%d", sblock->CharacterID);
-  printf ("%s(", newobj (tname, "TextField"));
+  printf ("%s();\n", newobj (tname, "TextField"));
 
+  printf ("%s(", methodcall (tname, "setFlags"));
   if (sblock->WordWrap)
     {
       printf ("SWFTEXTFIELD_WORDWRAP");
@@ -937,6 +942,7 @@ outputSWF_DEFINEEDITTEXT (SWF_Parserstruct * pblock)
       printf ("SWFTEXTFIELD_NOEDIT");
       notFirst = 1;
     }
+/* handled by setLength later on
   if (sblock->HasMaxLength)
     {
       if (notFirst)
@@ -944,6 +950,7 @@ outputSWF_DEFINEEDITTEXT (SWF_Parserstruct * pblock)
       printf ("SWFTEXTFIELD_HASLENGTH");
       notFirst = 1;
     }
+*/
   if (sblock->AutoSize)
     {
       if (notFirst)
@@ -997,10 +1004,14 @@ if( sblock->HasLayout ) {
         printf ("%s(" SQ , methodcall (tname, "addChars"));
         for(i=0;i<fi->fontcodearrsize;i++)
         {
-         #ifdef SWFPERL
+#ifdef SWFPERL
          if (fi->fontcodeptr[i]=='\'' || fi->fontcodeptr[i]=='\\' || fi->fontcodeptr[i]=='@' || fi->fontcodeptr[i]=='%' )
           printf ("\\");
-         #endif
+#endif
+#if defined(SWFPHP) || defined(SWFPYTHON)
+         if (fi->fontcodeptr[i]=='\'' || fi->fontcodeptr[i]=='\\')
+          printf ("\\");
+#endif
          if (fi->fontcodeptr[i]<256)
           printf ("%c",fi->fontcodeptr[i]);
         } 
@@ -1065,8 +1076,14 @@ if( sblock->HasLayout ) {
 void
 outputSWF_DEFINEFONT (SWF_Parserstruct * pblock)
 {
-  OUT_BEGIN_EMPTY (SWF_DEFINEFONT);
+  char fname[64];
+  OUT_BEGIN (SWF_DEFINEFONT);
 
+  sprintf (fname, "f%d", sblock->FontID);
+
+  printf ("\n" COMMSTART " Font %d (%d glyps)." COMMEND "\n",
+    sblock->FontID, sblock->NumGlyphs);
+  printf ("%s(\"font%d.fdb\" );\n", newobj (fname, "Font"), sblock->FontID);
 }
 
 /* save important part for later usage in outputSWF_DEFINETEXT(), outputSWF_DEFINETEXT2() */
@@ -1075,7 +1092,7 @@ static void saveFontInfo(int id,int numglyph,int *codetable,UI16 *ct16)
   struct FONTINFO *fi=fip;
 
   if (!fi) 
-    fi=fip=calloc(1,sizeof(struct FONTINFO));
+    fi=fip=fip_current=calloc(1,sizeof(struct FONTINFO));
   else
   {  
    while (fi->next)
@@ -1109,7 +1126,8 @@ outputSWF_DEFINEFONT2 (SWF_Parserstruct * pblock)
   sprintf (fname, "f%d", sblock->FontID);
   if (sblock->FontFlagsHasLayout || sblock->NumGlyphs)
   {
-   printf ("%s(\"%s.fdb\" );\n", newobj (fname, "Font"), sblock->FontName);
+   printf (COMMSTART " font name: %s" COMMEND "\n", sblock->FontName);
+   printf ("%s(\"font%d.fdb\" );\n", newobj (fname, "Font"), sblock->FontID);
    saveFontInfo(sblock->FontID,sblock->NumGlyphs,sblock->CodeTable,NULL);
   }
   else
@@ -1118,6 +1136,26 @@ outputSWF_DEFINEFONT2 (SWF_Parserstruct * pblock)
   }
 }
 
+void
+outputSWF_DEFINEFONT3 (SWF_Parserstruct * pblock)
+{
+  char fname[64];
+  OUT_BEGIN (SWF_DEFINEFONT3);
+
+  sprintf (fname, "f%d", sblock->FontID);
+
+  if (sblock->FontFlagsHasLayout || sblock->NumGlyphs)
+  {
+   printf (COMMSTART " font name: %s" COMMEND "\n", sblock->FontName);
+   printf ("%s(\"font%d.fdb\" );\n", newobj (fname, "Font"), sblock->FontID);
+   saveFontInfo(sblock->FontID,sblock->NumGlyphs,NULL,sblock->CodeTable);
+  }
+  else
+  {
+   printf ("%s(\"%s\" );\n", newobj (fname, "BrowserFont"), sblock->FontName);
+  }
+}
+
 void
 outputSWF_DEFINEFONTINFO (SWF_Parserstruct * pblock)
 {
@@ -1148,9 +1186,9 @@ outputSWF_DEFINELOSSLESS (SWF_Parserstruct * pblock)
   OUT_BEGIN (SWF_DEFINELOSSLESS);
 
   sprintf (name, "character%d", sblock->CharacterID);
-  printf ("\n" COMMSTART " Bitmap %d (lossless). To extract:" COMMEND "\n",
+  printf ("\n" COMMSTART " Bitmap %d (lossless). To extract: " COMMEND "\n",
     sblock->CharacterID);
-  printf ("\n" COMMSTART " swfextract -p %d -o %s.png $swf" COMMEND "\n",
+  printf (COMMSTART " swfextract -p %d -o %s.png $swf " COMMEND "\n",
     sblock->CharacterID, name);
   printf ("%s('%s.png');\n", newobj (name, "Bitmap"), name);
 
@@ -1166,7 +1204,7 @@ outputSWF_DEFINELOSSLESS2 (SWF_Parserstruct * pblock)
   sprintf (name, "character%d", sblock->CharacterID);
   printf ("\n" COMMSTART " Bitmap %d (lossless2). To extract:" COMMEND "\n",
     sblock->CharacterID);
-  printf ("\n" COMMSTART " swfextract -p %d -o %s.png $swf" COMMEND "\n",
+  printf (COMMSTART " swfextract -p %d -o %s.png $swf " COMMEND "\n",
     sblock->CharacterID, name);
   printf ("%s('%s.png');\n", newobj (name, "Bitmap"), name);
 }
@@ -1305,14 +1343,21 @@ outputSWF_TEXT_RECORD (SWF_TEXTRECORD *trec, int level,char *tname,char *buffer,
   {
     printf ("%s(%d, %d);\n", methodcall (tname, "moveTo"),trec->XOffset,trec->YOffset);
   }
-
+  if (trec->FontID) 
+  {
+    id=trec->FontID;
+  }
+  if (!trec->StyleFlagHasFont)				/* always check flag before use data */
+  {
+   fi = fip_current;					/* so cont w current font */
+   id = fi->fontcodeID;					/* trigger next if */
+  }
   while (fi)
   {
-   if (trec->FontID) 
-    id=trec->FontID;
    if (fi->fontcodeID==id)
    {
-    for(i=0;i<trec->GlyphCount && i<bsize ;i++)
+    fip_current=fi;					/* set current font */
+    for(i=0;i<trec->GlyphCount && i<bsize-1 ;i++)	/* byte n-1 will be terminator '\0' */
     {
      int off=(&(trec->GlyphEntries[i]))->GlyphIndex[0];
      if (off<fi->fontcodearrsize)
@@ -1321,7 +1366,7 @@ outputSWF_TEXT_RECORD (SWF_TEXTRECORD *trec, int level,char *tname,char *buffer,
       buffer[i]='?';		/* fallback to dummy A */
      /* printf ( COMMSTART "GlyphIndex[0] = %d  char = %d " COMMEND"\n",off,fi->fontcodeptr[off] ); */
     } 
-    buffer[i+1]='\0'; 
+    buffer[i]='\0'; 
     return;
    }
    else
@@ -1392,7 +1437,29 @@ outputSWF_DEFINEVIDEO (SWF_Parserstruct * pblock)
 void
 outputSWF_DEFINEVIDEOSTREAM (SWF_Parserstruct * pblock)
 {
-  OUT_BEGIN_EMPTY (SWF_DEFINEVIDEOSTREAM);
+  char name[32];
+
+  OUT_BEGIN (SWF_DEFINEVIDEOSTREAM);
+
+  sprintf (name, "character%d", sblock->CharacterID);
+
+  /* NOTE: Ming sets NumFrames = 65535 for empty movies.. */
+  if ( sblock->NumFrames && sblock->NumFrames != 65535 ) {
+    printf (COMMSTART " You'll need to extract video%d.flv " COMMEND "\n",
+            sblock->CharacterID);
+    printf ("%s('video%d.flv');", newobj (name, "VideoStream"),
+            sblock->CharacterID);
+  } else {
+    printf ("%s(); ", newobj (name, "VideoStream"));
+  }
+  printf (COMMSTART " %d frames advertised " COMMEND "\n", sblock->NumFrames);
+
+  printf ("%s(%d, %d);\n", methodcall (name, "setDimension"),
+    sblock->Width, sblock->Height);
+
+  /* We go manual and have SWF_VIDEOFRAME trigger a nextFrame */
+  printf ("%s(SWF_VIDEOSTREAM_MODE_MANUAL);\n",
+    methodcall (name, "setFrameMode"));
 
 }
 
@@ -1693,15 +1760,37 @@ outputSWF_INITACTION (SWF_Parserstruct * pblock)
 {
   OUT_BEGIN (SWF_INITACTION);
 
-  printf ("%s(%s(\"\n%s\") );\n", methodcall (spritenum?spritename:"m", "add"),
-	  newobj (NULL, "Action"), decompile5Action(sblock->numActions,sblock->Actions,0));
+  printf (COMMSTART
+    " Might be more appropriate to use addInitAction here" COMMEND "\n");
+
+  /* NOTE: the printf must be split in two cause newobj uses
+   *       a static buffer so can't be used twice to return
+   *       different strings
+   */
+
+  printf ("%s(%s(",
+          methodcall (spritenum?spritename:"m", "add"),
+          newobj (NULL, "InitAction"));
+
+  /* TODO: add SpriteID ? */
+
+  printf ("%s(\"%s\")));\n",
+          newobj (NULL, "Action"),
+          decompile5Action(sblock->numActions,sblock->Actions,0));
 }
 
 void
 outputSWF_VIDEOFRAME (SWF_Parserstruct * pblock)
 {
-  OUT_BEGIN_EMPTY (SWF_VIDEOFRAME);
+  char name[32];
 
+  OUT_BEGIN (SWF_VIDEOFRAME);
+
+  sprintf (name, "character%d", sblock->StreamID);
+
+  printf (COMMSTART " Frame %d of stream %d " COMMEND "\n",
+    sblock->FrameNum, sblock->StreamID);
+  printf ("%s();\n", methodcall (name, "nextFrame"));
 }
 
 void
@@ -1827,7 +1916,7 @@ outputHeader (struct Movie *m)
   if( m->version != 5 ) 
 	printf ("Ming_useSWFVersion(%d);\n\n", m->version);
   printf ("%s();\n\n", newobj ("m", "Movie"));
-  printf (COMMSTART "add setscale here" COMMEND "\n");
+  printf ("Ming_setScale(1.0);\n");
 #endif
 #ifdef SWFPLUSPLUS
   printf ("#include <mingpp.h>\n");
